@@ -14,30 +14,15 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.
 
-   You should have received a copy of the GNU Library General Public
-   License along with the Gnome Library; see the file COPYING.LIB.  If not,
-   write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   You should have received a copy of the GNU General Public License along
+   with this program; if not, write to the Free Software Foundation, Inc.,
+   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
    Authors: Ramiro Estrugo <ramiro@eazel.com>
 */
 
 #include <config.h>
-//#include "gnome-i18nP.h"
 #include "gnome-two-password-dialog.h"
-#include <gtk/gtkbox.h>
-#include <gtk/gtkcheckbutton.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtkhbox.h>
-#include <gtk/gtkimage.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkmain.h>
-#include <gtk/gtksignal.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtktable.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkradiobutton.h>
-#include <gtk/gtkstock.h>
 
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
@@ -170,6 +155,26 @@ gnome_two_password_dialog_finalize (GObject *object)
 		(* G_OBJECT_CLASS (parent_class)->finalize) (object);
 }
 
+static gboolean
+is_visible (GtkWidget *w)
+{
+#if GTK_CHECK_VERSION(2,20,0)
+	return gtk_widget_get_visible (w);
+#else
+	return GTK_WIDGET_VISIBLE (w);
+#endif
+}
+
+static gboolean
+is_sensitive (GtkWidget *w)
+{
+#if GTK_CHECK_VERSION(2,18,0)
+	return gtk_widget_get_sensitive (w);
+#else
+	return GTK_WIDGET_SENSITIVE (w);
+#endif
+}
+
 /* GtkDialog callbacks */
 static void
 dialog_show_callback (GtkWidget *widget, gpointer callback_data)
@@ -178,15 +183,15 @@ dialog_show_callback (GtkWidget *widget, gpointer callback_data)
 
 	password_dialog = GNOME_TWO_PASSWORD_DIALOG (callback_data);
 
-	if (GTK_WIDGET_VISIBLE (password_dialog->details->username_entry) &&
-	    !password_dialog->details->readonly_username) {
+	if (   is_visible (password_dialog->details->username_entry)
+	    && !password_dialog->details->readonly_username) {
 		gtk_widget_grab_focus (password_dialog->details->username_entry);
-	} else if (GTK_WIDGET_VISIBLE (password_dialog->details->domain_entry) &&
-		   !password_dialog->details->readonly_domain) {
+	} else if (   is_visible (password_dialog->details->domain_entry)
+	           && !password_dialog->details->readonly_domain) {
 		gtk_widget_grab_focus (password_dialog->details->domain_entry);
-	} else if (GTK_WIDGET_VISIBLE (password_dialog->details->password_entry)) {
+	} else if (is_visible (password_dialog->details->password_entry)) {
 		gtk_widget_grab_focus (password_dialog->details->password_entry);
-	} else if (GTK_WIDGET_VISIBLE (password_dialog->details->password_entry_secondary)) {
+	} else if (is_visible (password_dialog->details->password_entry_secondary)) {
 		gtk_widget_grab_focus (password_dialog->details->password_entry_secondary);
 	}
 }
@@ -280,14 +285,14 @@ username_entry_activate (GtkWidget *widget, GtkWidget *dialog)
 
 	password_dialog = GNOME_TWO_PASSWORD_DIALOG (dialog);
 	
-	if (GTK_WIDGET_VISIBLE (password_dialog->details->domain_entry) &&
-	    GTK_WIDGET_SENSITIVE (password_dialog->details->domain_entry))
+	if (is_visible (password_dialog->details->domain_entry) &&
+	    is_sensitive (password_dialog->details->domain_entry))
 		gtk_widget_grab_focus (password_dialog->details->domain_entry);
-	else if (GTK_WIDGET_VISIBLE (password_dialog->details->password_entry) &&
-		 GTK_WIDGET_SENSITIVE (password_dialog->details->password_entry))
+	else if (is_visible (password_dialog->details->password_entry) &&
+		 is_sensitive (password_dialog->details->password_entry))
 		gtk_widget_grab_focus (password_dialog->details->password_entry);
-	else if (GTK_WIDGET_VISIBLE (password_dialog->details->password_entry_secondary) &&
-		 GTK_WIDGET_SENSITIVE (password_dialog->details->password_entry_secondary))
+	else if (is_visible (password_dialog->details->password_entry_secondary) &&
+		 is_sensitive (password_dialog->details->password_entry_secondary))
 		gtk_widget_grab_focus (password_dialog->details->password_entry_secondary);
 }
 
@@ -298,11 +303,11 @@ domain_entry_activate (GtkWidget *widget, GtkWidget *dialog)
 
 	password_dialog = GNOME_TWO_PASSWORD_DIALOG (dialog);
 	
-	if (GTK_WIDGET_VISIBLE (password_dialog->details->password_entry) &&
-	    GTK_WIDGET_SENSITIVE (password_dialog->details->password_entry))
+	if (is_visible (password_dialog->details->password_entry) &&
+	    is_sensitive (password_dialog->details->password_entry))
 		gtk_widget_grab_focus (password_dialog->details->password_entry);
-	else if (GTK_WIDGET_VISIBLE (password_dialog->details->password_entry_secondary) &&
-		 GTK_WIDGET_SENSITIVE (password_dialog->details->password_entry_secondary))
+	else if (is_visible (password_dialog->details->password_entry_secondary) &&
+		 is_sensitive (password_dialog->details->password_entry_secondary))
 		gtk_widget_grab_focus (password_dialog->details->password_entry_secondary);
 }
 
@@ -324,6 +329,7 @@ gnome_two_password_dialog_new (const char	*dialog_title,
 	GtkWidget *main_vbox;
 	GtkWidget *dialog_icon;
 	GSList *group;
+	GtkBox *content, *action_area;
 
 	password_dialog = GNOME_TWO_PASSWORD_DIALOG (gtk_widget_new (gnome_two_password_dialog_get_type (), NULL));
 	dialog = GTK_DIALOG (password_dialog);
@@ -337,12 +343,15 @@ gnome_two_password_dialog_new (const char	*dialog_title,
 				NULL);
 	gtk_dialog_set_default_response (GTK_DIALOG (password_dialog), GTK_RESPONSE_OK);
 
+	content = GTK_BOX (gtk_dialog_get_content_area (GTK_DIALOG (dialog)));
+	action_area = GTK_BOX (gtk_dialog_get_action_area (GTK_DIALOG (dialog)));
+
 	/* Setup the dialog */
 	gtk_dialog_set_has_separator (dialog, FALSE);
         gtk_container_set_border_width (GTK_CONTAINER (dialog), 5);
-        gtk_box_set_spacing (GTK_BOX (dialog->vbox), 2); /* 2 * 5 + 2 = 12 */
-        gtk_container_set_border_width (GTK_CONTAINER (dialog->action_area), 5);
-        gtk_box_set_spacing (GTK_BOX (dialog->action_area), 6);
+	gtk_box_set_spacing (content, 2); /* 2 * 5 + 2 = 12 */
+	gtk_container_set_border_width (GTK_CONTAINER (action_area), 5);
+	gtk_box_set_spacing (action_area, 6);
 
  	gtk_window_set_position (GTK_WINDOW (password_dialog), GTK_WIN_POS_CENTER);
 	gtk_window_set_modal (GTK_WINDOW (password_dialog), TRUE);
@@ -459,13 +468,9 @@ gnome_two_password_dialog_new (const char	*dialog_title,
 
 	gtk_box_pack_start (GTK_BOX (hbox), main_vbox, FALSE, FALSE, 0);
 
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (password_dialog)->vbox),
-			    hbox,
-			    TRUE,	/* expand */
-			    TRUE,	/* fill */
-			    0);       	/* padding */
+	gtk_box_pack_start (content, hbox, TRUE, TRUE, 0);
 	
-	gtk_widget_show_all (GTK_DIALOG (password_dialog)->vbox);
+	gtk_widget_show_all (GTK_WIDGET (content));
 
 	password_dialog->details->remember_session_button =
 		gtk_check_button_new_with_mnemonic (_("_Remember password for this session"));
