@@ -2,7 +2,7 @@
 /* nm-openvpn-service - openvpn integration with NetworkManager
  *
  * Copyright (C) 2005 - 2008 Tim Niemueller <tim@niemueller.de>
- * Copyright (C) 2005 - 2008 Dan Williams <dcbw@redhat.com>
+ * Copyright (C) 2005 - 2010 Dan Williams <dcbw@redhat.com>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -52,6 +52,14 @@
 
 #include "nm-openvpn-service.h"
 #include "nm-utils.h"
+#include "common/utils.h"
+
+#if !defined(DIST_VERSION)
+# define DIST_VERSION VERSION
+#endif
+
+static gboolean debug = FALSE;
+static GMainLoop *loop = NULL;
 
 #define NM_OPENVPN_HELPER_PATH		LIBEXECDIR"/nm-openvpn-service-openvpn-helper"
 
@@ -63,6 +71,8 @@ typedef struct {
 	char *username;
 	char *password;
 	char *priv_key_pass;
+	char *proxy_username;
+	char *proxy_password;
 	GIOChannel *socket_channel;
 	guint socket_channel_eventid;
 } NMOpenvpnPluginIOData;
@@ -89,20 +99,28 @@ static ValidProperty valid_properties[] = {
 	{ NM_OPENVPN_KEY_CIPHER,               G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_COMP_LZO,             G_TYPE_BOOLEAN, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_CONNECTION_TYPE,      G_TYPE_STRING, 0, 0, FALSE },
-	{ NM_OPENVPN_KEY_TAP_DEV,              G_TYPE_BOOLEAN, 0, 0, FALSE },
+	{ NM_OPENVPN_KEY_FRAGMENT_SIZE,        G_TYPE_INT, 0, G_MAXINT, FALSE },
 	{ NM_OPENVPN_KEY_KEY,                  G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_LOCAL_IP,             G_TYPE_STRING, 0, 0, TRUE },
+	{ NM_OPENVPN_KEY_MSSFIX,               G_TYPE_BOOLEAN, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_PROTO_TCP,            G_TYPE_BOOLEAN, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_PORT,                 G_TYPE_INT, 1, 65535, FALSE },
+	{ NM_OPENVPN_KEY_PROXY_TYPE,           G_TYPE_STRING, 0, 0, FALSE },
+	{ NM_OPENVPN_KEY_PROXY_SERVER,         G_TYPE_STRING, 0, 0, FALSE },
+	{ NM_OPENVPN_KEY_PROXY_PORT,           G_TYPE_INT, 1, 65535, FALSE },
+	{ NM_OPENVPN_KEY_PROXY_RETRY,          G_TYPE_BOOLEAN, 0, 0, FALSE },
+	{ NM_OPENVPN_KEY_HTTP_PROXY_USERNAME,  G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_REMOTE,               G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_REMOTE_IP,            G_TYPE_STRING, 0, 0, TRUE },
+	{ NM_OPENVPN_KEY_RENEG_SECONDS,        G_TYPE_INT, 0, G_MAXINT, FALSE },
 	{ NM_OPENVPN_KEY_STATIC_KEY,           G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_STATIC_KEY_DIRECTION, G_TYPE_INT, 0, 1, FALSE },
 	{ NM_OPENVPN_KEY_TA,                   G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_TA_DIR,               G_TYPE_INT, 0, 1, FALSE },
-	{ NM_OPENVPN_KEY_USERNAME,             G_TYPE_STRING, 0, 0, FALSE },
-	{ NM_OPENVPN_KEY_RENEG_SECONDS,        G_TYPE_INT, 0, G_MAXINT, FALSE },
+	{ NM_OPENVPN_KEY_TAP_DEV,              G_TYPE_BOOLEAN, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_TLS_REMOTE,	       G_TYPE_STRING, 0, 0, FALSE },
+	{ NM_OPENVPN_KEY_TUNNEL_MTU,           G_TYPE_INT, 0, G_MAXINT, FALSE },
+	{ NM_OPENVPN_KEY_USERNAME,             G_TYPE_STRING, 0, 0, FALSE },
 	{ NULL,                                G_TYPE_NONE, FALSE }
 };
 
@@ -110,6 +128,7 @@ static ValidProperty valid_secrets[] = {
 	{ NM_OPENVPN_KEY_PASSWORD,             G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_CERTPASS,             G_TYPE_STRING, 0, 0, FALSE },
 	{ NM_OPENVPN_KEY_NOSECRET,             G_TYPE_STRING, 0, 0, FALSE },
+	{ NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD,  G_TYPE_STRING, 0, 0, FALSE },
 	{ NULL,                                G_TYPE_NONE, FALSE }
 };
 
@@ -268,13 +287,28 @@ nm_openvpn_disconnect_management_socket (NMOpenvpnPlugin *plugin)
 	if (!io_data)
 		return;
 
-	g_source_remove (io_data->socket_channel_eventid);
-	g_io_channel_shutdown (io_data->socket_channel, FALSE, NULL);
-	g_io_channel_unref (io_data->socket_channel);
+	if (io_data->socket_channel_eventid)
+		g_source_remove (io_data->socket_channel_eventid);
+	if (io_data->socket_channel) {
+		g_io_channel_shutdown (io_data->socket_channel, FALSE, NULL);
+		g_io_channel_unref (io_data->socket_channel);
+	}
 
 	g_free (io_data->username);
+	g_free (io_data->proxy_username);
+
+	if (io_data->password)
+		memset (io_data->password, 0, strlen (io_data->password));
 	g_free (io_data->password);
 
+	if (io_data->priv_key_pass)
+		memset (io_data->priv_key_pass, 0, strlen (io_data->priv_key_pass));
+	g_free (io_data->priv_key_pass);
+
+	if (io_data->proxy_password)
+		memset (io_data->proxy_password, 0, strlen (io_data->proxy_password));
+	g_free (io_data->proxy_password);
+
 	g_free (priv->io_data);
 	priv->io_data = NULL;
 }
@@ -328,6 +362,33 @@ get_detail (const char *input, const char *prefix)
 	return ret;
 }
 
+static void
+write_user_pass (GIOChannel *channel,
+                 const char *authtype,
+                 const char *user,
+                 const char *pass)
+{
+	char *quser, *qpass, *buf;
+
+	/* Quote strings passed back to openvpn */
+	quser = ovpn_quote_string (user);
+	qpass = ovpn_quote_string (pass);
+	buf = g_strdup_printf ("username \"%s\" \"%s\"\n"
+	                       "password \"%s\" \"%s\"\n",
+	                       authtype, quser,
+	                       authtype, qpass);
+	memset (qpass, 0, strlen (qpass));
+	g_free (qpass);
+	g_free (quser);
+
+	/* Will always write everything in blocking channels (on success) */
+	g_io_channel_write_chars (channel, buf, strlen (buf), NULL, NULL);
+	g_io_channel_flush (channel, NULL);
+
+	memset (buf, 0, strlen (buf));
+	g_free (buf);
+}
+
 static gboolean
 handle_management_socket (NMVPNPlugin *plugin,
                           GIOChannel *source,
@@ -337,7 +398,6 @@ handle_management_socket (NMVPNPlugin *plugin,
 	NMOpenvpnPluginIOData *io_data = NM_OPENVPN_PLUGIN_GET_PRIVATE (plugin)->io_data;
 	gboolean again = TRUE;
 	char *str = NULL, *auth = NULL, *buf;
-	gsize written;
 
 	if (!(condition & G_IO_IN))
 		return TRUE;
@@ -351,25 +411,9 @@ handle_management_socket (NMVPNPlugin *plugin,
 	auth = get_detail (str, ">PASSWORD:Need '");
 	if (auth) {
 		if (strcmp (auth, "Auth") == 0) {
-			if (io_data->username != NULL && io_data->password != NULL) {
-				char *quser, *qpass;
-
-				/* Quote strings passed back to openvpn */
-				quser = ovpn_quote_string (io_data->username);
-				qpass = ovpn_quote_string (io_data->password);
-				buf = g_strdup_printf ("username \"%s\" \"%s\"\n"
-				                       "password \"%s\" \"%s\"\n",
-				                       auth, quser,
-				                       auth, qpass);
-				memset (qpass, 0, strlen (qpass));
-				g_free (qpass);
-				g_free (quser);
-
-				/* Will always write everything in blocking channels (on success) */
-				g_io_channel_write_chars (source, buf, strlen (buf), &written, NULL);
-				g_io_channel_flush (source, NULL);
-				g_free (buf);
-			} else
+			if (io_data->username != NULL && io_data->password != NULL)
+				write_user_pass (source, auth, io_data->username, io_data->password);
+			else
 				nm_warning ("Auth requested but one of username or password is missing");
 		} else if (!strcmp (auth, "Private Key")) {
 			if (io_data->priv_key_pass) {
@@ -382,11 +426,16 @@ handle_management_socket (NMVPNPlugin *plugin,
 				g_free (qpass);
 
 				/* Will always write everything in blocking channels (on success) */
-				g_io_channel_write_chars (source, buf, strlen (buf), &written, NULL);
+				g_io_channel_write_chars (source, buf, strlen (buf), NULL, NULL);
 				g_io_channel_flush (source, NULL);
 				g_free (buf);
 			} else
 				nm_warning ("Certificate password requested but private key password == NULL");
+		} else if (strcmp (auth, "HTTP Proxy") == 0) {
+			if (io_data->proxy_username != NULL && io_data->proxy_password != NULL)
+				write_user_pass (source, auth, io_data->proxy_username, io_data->proxy_password);
+			else
+				nm_warning ("HTTP Proxy auth requested but either proxy username or password is missing");
 		} else {
 			nm_warning ("No clue what to send for username/password request for '%s'", auth);
 			if (out_failure)
@@ -552,8 +601,14 @@ validate_auth (const char *auth)
 {
 	if (auth) {
 		if (   !strcmp (auth, NM_OPENVPN_AUTH_NONE)
+		    || !strcmp (auth, NM_OPENVPN_AUTH_RSA_MD4)
 		    || !strcmp (auth, NM_OPENVPN_AUTH_MD5)
-		    || !strcmp (auth, NM_OPENVPN_AUTH_SHA1))
+		    || !strcmp (auth, NM_OPENVPN_AUTH_SHA1)
+		    || !strcmp (auth, NM_OPENVPN_AUTH_SHA224)
+		    || !strcmp (auth, NM_OPENVPN_AUTH_SHA256)
+		    || !strcmp (auth, NM_OPENVPN_AUTH_SHA384)
+		    || !strcmp (auth, NM_OPENVPN_AUTH_SHA512)
+		    || !strcmp (auth, NM_OPENVPN_AUTH_RIPEMD160))
 			return TRUE;
 	}
 	return FALSE;
@@ -627,6 +682,43 @@ add_openvpn_arg_int (GPtrArray *args, const char *arg)
 	return TRUE;
 }
 
+static void
+add_cert_args (GPtrArray *args, NMSettingVPN *s_vpn)
+{
+	const char *ca, *cert, *key;
+
+	g_return_if_fail (args != NULL);
+	g_return_if_fail (s_vpn != NULL);
+
+	ca = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CA);
+	cert = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CERT);
+	key = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_KEY);
+
+	if (   ca && strlen (ca)
+	    && cert && strlen (cert)
+	    && key && strlen (key)
+	    && !strcmp (ca, cert)
+	    && !strcmp (ca, key)) {
+		add_openvpn_arg (args, "--pkcs12");
+		add_openvpn_arg (args, ca);
+	} else {
+		if (ca && strlen (ca)) {
+			add_openvpn_arg (args, "--ca");
+			add_openvpn_arg (args, ca);
+		}
+
+		if (cert && strlen (cert)) {
+			add_openvpn_arg (args, "--cert");
+			add_openvpn_arg (args, cert);
+		}
+
+		if (key && strlen (key)) {
+			add_openvpn_arg (args, "--key");
+			add_openvpn_arg (args, key);
+		}
+	}
+}
+
 static gboolean
 nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
                                  NMSettingVPN *s_vpn,
@@ -634,7 +726,7 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
                                  GError **error)
 {
 	NMOpenvpnPluginPrivate *priv = NM_OPENVPN_PLUGIN_GET_PRIVATE (plugin);
-	const char *openvpn_binary, *auth, *connection_type, *tmp;
+	const char *openvpn_binary, *auth, *connection_type, *tmp, *tmp2, *tmp3, *tmp4;
 	GPtrArray *args;
 	GSource *openvpn_watch;
 	GPid pid;
@@ -682,6 +774,34 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 		add_openvpn_arg (args, tmp);
 	}
 
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_PROXY_TYPE);
+	tmp2 = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_PROXY_SERVER);
+	tmp3 = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_PROXY_PORT);
+	tmp4 = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_PROXY_RETRY);
+	if (tmp && strlen (tmp) && tmp2 && strlen (tmp2)) {
+		if (!strcmp (tmp, "http")) {
+			add_openvpn_arg (args, "--http-proxy");
+			add_openvpn_arg (args, tmp2);
+			add_openvpn_arg (args, tmp3 ? tmp3 : "8080");
+			add_openvpn_arg (args, "auto");  /* Automatic proxy auth method detection */
+			if (tmp4)
+				add_openvpn_arg (args, "--http-proxy-retry");
+		} else if (!strcmp (tmp, "socks")) {
+			add_openvpn_arg (args, "--socks-proxy");
+			add_openvpn_arg (args, tmp2);
+			add_openvpn_arg (args, tmp3 ? tmp3 : "1080");
+			if (tmp4)
+				add_openvpn_arg (args, "--socks-proxy-retry");
+		} else {
+			g_set_error (error,
+				         NM_VPN_PLUGIN_ERROR,
+				         NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
+				         "Invalid proxy type '%s'.",
+				         tmp);
+			return FALSE;
+		}
+	}
+
 	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_COMP_LZO);
 	if (tmp && !strcmp (tmp, "yes"))
 		add_openvpn_arg (args, "--comp-lzo");
@@ -769,7 +889,7 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 		}
 	}
 
-	if (getenv ("OPENVPN_DEBUG")) {
+	if (debug) {
 		add_openvpn_arg (args, "--verb");
 		add_openvpn_arg (args, "10");
 	} else {
@@ -778,6 +898,42 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 		add_openvpn_arg (args, "nm-openvpn");
 	}
 
+	/* TUN MTU size */
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_TUNNEL_MTU);
+	if (tmp && strlen (tmp)) {
+		add_openvpn_arg (args, "--tun-mtu");
+		if (!add_openvpn_arg_int (args, tmp)) {
+			g_set_error (error,
+			             NM_VPN_PLUGIN_ERROR,
+			             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
+			             "Invalid TUN MTU size '%s'.",
+			             tmp);
+			free_openvpn_args (args);
+			return FALSE;
+		}
+	}
+
+	/* fragment size */
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_FRAGMENT_SIZE);
+	if (tmp && strlen (tmp)) {
+		add_openvpn_arg (args, "--fragment");
+		if (!add_openvpn_arg_int (args, tmp)) {
+			g_set_error (error,
+			             NM_VPN_PLUGIN_ERROR,
+			             NM_VPN_PLUGIN_ERROR_BAD_ARGUMENTS,
+			             "Invalid fragment size '%s'.",
+			             tmp);
+			free_openvpn_args (args);
+			return FALSE;
+		}
+	}
+
+	/* mssfix */
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_MSSFIX);
+	if (tmp && !strcmp (tmp, "yes")) {
+		add_openvpn_arg (args, "--mssfix");
+	}
+
 	/* Punch script security in the face; this option was added to OpenVPN 2.1-rc9
 	 * and defaults to disallowing any scripts, a behavior change from previous
 	 * versions.
@@ -787,7 +943,10 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 
 	/* Up script, called when connection has been established or has been restarted */
 	add_openvpn_arg (args, "--up");
-	add_openvpn_arg (args, NM_OPENVPN_HELPER_PATH);
+	if (debug)
+		add_openvpn_arg (args, NM_OPENVPN_HELPER_PATH " --helper-debug");
+	else
+		add_openvpn_arg (args, NM_OPENVPN_HELPER_PATH);
 	add_openvpn_arg (args, "--up-restart");
 
 	/* Keep key and tun if restart is needed */
@@ -802,30 +961,14 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 	/* Query on the management socket for user/pass */
 	add_openvpn_arg (args, "--management-query-passwords");
 
-	/* do not let openvpn setup routes, NM will handle it */
+	/* do not let openvpn setup routes or addresses, NM will handle it */
 	add_openvpn_arg (args, "--route-noexec");
+	add_openvpn_arg (args, "--ifconfig-noexec");
 
 	/* Now append configuration options which are dependent on the configuration type */
 	if (!strcmp (connection_type, NM_OPENVPN_CONTYPE_TLS)) {
 		add_openvpn_arg (args, "--client");
-
-		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CA);
-		if (tmp && strlen (tmp)) {
-			add_openvpn_arg (args, "--ca");
-			add_openvpn_arg (args, tmp);
-		}
-
-		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CERT);
-		if (tmp && strlen (tmp)) {
-			add_openvpn_arg (args, "--cert");
-			add_openvpn_arg (args, tmp);
-		}
-
-		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_KEY);
-		if (tmp && strlen (tmp)) {
-			add_openvpn_arg (args, "--key");
-			add_openvpn_arg (args, tmp);
-		}
+		add_cert_args (args, s_vpn);
 	} else if (!strcmp (connection_type, NM_OPENVPN_CONTYPE_STATIC_KEY)) {
 		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_STATIC_KEY);
 		if (tmp && strlen (tmp)) {
@@ -877,25 +1020,7 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 		}
 	} else if (!strcmp (connection_type, NM_OPENVPN_CONTYPE_PASSWORD_TLS)) {
 		add_openvpn_arg (args, "--client");
-
-		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CA);
-		if (tmp && strlen (tmp)) {
-			add_openvpn_arg (args, "--ca");
-			add_openvpn_arg (args, tmp);
-		}
-
-		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CERT);
-		if (tmp && strlen (tmp)) {
-			add_openvpn_arg (args, "--cert");
-			add_openvpn_arg (args, tmp);
-		}
-
-		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_KEY);
-		if (tmp && strlen (tmp)) {
-			add_openvpn_arg (args, "--key");
-			add_openvpn_arg (args, tmp);
-		}
-
+		add_cert_args (args, s_vpn);
 		/* Use user/path authentication */
 		add_openvpn_arg (args, "--auth-user-pass");
 	} else {
@@ -932,24 +1057,28 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 	*/
 	if (   !strcmp (connection_type, NM_OPENVPN_CONTYPE_TLS)
 	    || !strcmp (connection_type, NM_OPENVPN_CONTYPE_PASSWORD)
-	    || !strcmp (connection_type, NM_OPENVPN_CONTYPE_PASSWORD_TLS)) {
-		NMOpenvpnPluginIOData *io_data;
+	    || !strcmp (connection_type, NM_OPENVPN_CONTYPE_PASSWORD_TLS)
+	    || nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_HTTP_PROXY_USERNAME)) {
 
-		io_data = g_new0 (NMOpenvpnPluginIOData, 1);
+		priv->io_data = g_malloc0 (sizeof (NMOpenvpnPluginIOData));
 
 		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_USERNAME);
-		io_data->username = tmp ? g_strdup (tmp) : NULL;
+		priv->io_data->username = tmp ? g_strdup (tmp) : NULL;
 		/* Use the default username if it wasn't overridden by the user */
-		if (!io_data->username && default_username)
-			io_data->username = g_strdup (default_username);
+		if (!priv->io_data->username && default_username)
+			priv->io_data->username = g_strdup (default_username);
 
 		tmp = nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_PASSWORD);
-		io_data->password = tmp ? g_strdup (tmp) : NULL;
+		priv->io_data->password = tmp ? g_strdup (tmp) : NULL;
 
 		tmp = nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_CERTPASS);
-		io_data->priv_key_pass = tmp ? g_strdup (tmp) : NULL;
+		priv->io_data->priv_key_pass = tmp ? g_strdup (tmp) : NULL;
 
-		priv->io_data = io_data;
+		tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_HTTP_PROXY_USERNAME);
+		priv->io_data->proxy_username = tmp ? g_strdup (tmp) : NULL;
+
+		tmp = nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD);
+		priv->io_data->proxy_password = tmp ? g_strdup (tmp) : NULL;
 
 		nm_openvpn_schedule_connect_timer (plugin);
 	}
@@ -957,6 +1086,50 @@ nm_openvpn_start_openvpn_binary (NMOpenvpnPlugin *plugin,
 	return TRUE;
 }
 
+static const char *
+check_need_secrets (NMSettingVPN *s_vpn, gboolean *need_secrets)
+{
+	const char *tmp, *key, *ctype;
+
+	g_return_val_if_fail (s_vpn != NULL, FALSE);
+	g_return_val_if_fail (need_secrets != NULL, FALSE);
+
+	*need_secrets = FALSE;
+
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CONNECTION_TYPE);
+	ctype = validate_connection_type (tmp);
+	if (!ctype)
+		return NULL;
+
+	if (!strcmp (ctype, NM_OPENVPN_CONTYPE_PASSWORD_TLS)) {
+		/* Will require a password and maybe private key password */
+		key = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_KEY);
+		if (is_encrypted (key) && !nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_CERTPASS))
+			*need_secrets = TRUE;
+
+		if (!nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_PASSWORD))
+			*need_secrets = TRUE;
+	} else if (!strcmp (ctype, NM_OPENVPN_CONTYPE_PASSWORD)) {
+		/* Will require a password */
+		if (!nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_PASSWORD))
+			*need_secrets = TRUE;
+	} else if (!strcmp (ctype, NM_OPENVPN_CONTYPE_TLS)) {
+		/* May require private key password */
+		key = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_KEY);
+		if (is_encrypted (key) && !nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_CERTPASS))
+			*need_secrets = TRUE;
+	} else {
+		/* Static key doesn't need passwords */
+	}
+
+	/* HTTP Proxy might require a password; assume so if there's an HTTP proxy username */
+	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_HTTP_PROXY_USERNAME);
+	if (tmp && !nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_HTTP_PROXY_PASSWORD))
+		*need_secrets = TRUE;
+
+	return ctype;
+}
+
 static gboolean
 real_connect (NMVPNPlugin   *plugin,
               NMConnection  *connection,
@@ -965,7 +1138,7 @@ real_connect (NMVPNPlugin   *plugin,
 	NMSettingVPN *s_vpn;
 	const char *connection_type;
 	const char *user_name;
-	const char *tmp;
+	gboolean need_secrets;
 
 	s_vpn = NM_SETTING_VPN (nm_connection_get_setting (connection, NM_TYPE_SETTING_VPN));
 	if (!s_vpn) {
@@ -977,11 +1150,8 @@ real_connect (NMVPNPlugin   *plugin,
 		return FALSE;
 	}
 
-	user_name = nm_setting_vpn_get_user_name (s_vpn);
-	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CONNECTION_TYPE);
-	
-	connection_type = validate_connection_type (tmp);
-	
+	/* Check if we need secrets and validate the connection type */
+	connection_type = check_need_secrets (s_vpn, &need_secrets);
 	if (!connection_type) {
 		g_set_error (error,
 		             NM_VPN_PLUGIN_ERROR,
@@ -989,7 +1159,9 @@ real_connect (NMVPNPlugin   *plugin,
 		             "%s",
 		             "Invalid connection type.");
 		return FALSE;
-	}	
+	}
+
+	user_name = nm_setting_vpn_get_user_name (s_vpn);
 
 	/* Need a username for any password-based connection types */
 	if (   !strcmp (connection_type, NM_OPENVPN_CONTYPE_PASSWORD_TLS)
@@ -1008,8 +1180,8 @@ real_connect (NMVPNPlugin   *plugin,
 	if (!nm_openvpn_properties_validate (s_vpn, error))
 		return FALSE;
 
-	/* Static Key doesn't need secrets; the rest do */
-	if (strcmp (connection_type, NM_OPENVPN_CONTYPE_STATIC_KEY)) {
+	/* Validate secrets */
+	if (need_secrets) {
 		if (!nm_openvpn_secrets_validate (s_vpn, error))
 			return FALSE;
 	}
@@ -1030,7 +1202,6 @@ real_need_secrets (NMVPNPlugin *plugin,
 	NMSettingVPN *s_vpn;
 	const char *connection_type;
 	gboolean need_secrets = FALSE;
-	const char *tmp;
 
 	g_return_val_if_fail (NM_IS_VPN_PLUGIN (plugin), FALSE);
 	g_return_val_if_fail (NM_IS_CONNECTION (connection), FALSE);
@@ -1045,9 +1216,7 @@ real_need_secrets (NMVPNPlugin *plugin,
 		return FALSE;
 	}
 
-	tmp = nm_setting_vpn_get_data_item (s_vpn, NM_OPENVPN_KEY_CONNECTION_TYPE);
-	connection_type = validate_connection_type (tmp);
-	
+	connection_type = check_need_secrets (s_vpn, &need_secrets);
 	if (!connection_type) {
 		g_set_error (error,
 		             NM_VPN_PLUGIN_ERROR,
@@ -1057,23 +1226,6 @@ real_need_secrets (NMVPNPlugin *plugin,
 		return FALSE;
 	}
 
-	if (!strcmp (connection_type, NM_OPENVPN_CONTYPE_PASSWORD_TLS)) {
-		/* Will require a password and maybe private key password */
-		if (!nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_CERTPASS))
-			need_secrets = TRUE;
-
-		if (!nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_PASSWORD))
-			need_secrets = TRUE;
-	} else if (!strcmp (connection_type, NM_OPENVPN_CONTYPE_PASSWORD)) {
-		/* Will require a password */
-		if (!nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_PASSWORD))
-			need_secrets = TRUE;
-	} else if (!strcmp (connection_type, NM_OPENVPN_CONTYPE_TLS)) {
-		/* May require private key password */
-		if (!nm_setting_vpn_get_secret (s_vpn, NM_OPENVPN_KEY_CERTPASS))
-			need_secrets = TRUE;
-	}
-
 	if (need_secrets)
 		*setting_name = NM_SETTING_VPN_SETTING_NAME;
 
@@ -1169,6 +1321,27 @@ nm_openvpn_plugin_new (void)
 	return plugin;
 }
 
+static void
+signal_handler (int signo)
+{
+	if (signo == SIGINT || signo == SIGTERM)
+		g_main_loop_quit (loop);
+}
+
+static void
+setup_signals (void)
+{
+	struct sigaction action;
+	sigset_t mask;
+
+	sigemptyset (&mask);
+	action.sa_handler = signal_handler;
+	action.sa_mask = mask;
+	action.sa_flags = 0;
+	sigaction (SIGTERM,  &action, NULL);
+	sigaction (SIGINT,  &action, NULL);
+}
+
 static void
 quit_mainloop (NMVPNPlugin *plugin, gpointer user_data)
 {
@@ -1179,10 +1352,36 @@ int
 main (int argc, char *argv[])
 {
 	NMOpenvpnPlugin *plugin;
-	GMainLoop *main_loop;
+	gboolean persist = FALSE;
+	GOptionContext *opt_ctx = NULL;
+
+	GOptionEntry options[] = {
+		{ "persist", 0, 0, G_OPTION_ARG_NONE, &persist, "Don't quit when VPN connection terminates", NULL },
+		{ "debug", 0, 0, G_OPTION_ARG_NONE, &debug, "Enable verbose debug logging (may expose passwords)", NULL },
+		{NULL}
+	};
 
 	g_type_init ();
 
+	/* Parse options */
+	opt_ctx = g_option_context_new ("");
+	g_option_context_set_translation_domain (opt_ctx, "UTF-8");
+	g_option_context_set_ignore_unknown_options (opt_ctx, FALSE);
+	g_option_context_set_help_enabled (opt_ctx, TRUE);
+	g_option_context_add_main_entries (opt_ctx, options, NULL);
+
+	g_option_context_set_summary (opt_ctx,
+		"nm-vpnc-service provides integrated OpenVPN capability to NetworkManager.");
+
+	g_option_context_parse (opt_ctx, &argc, &argv, NULL);
+	g_option_context_free (opt_ctx);
+
+	if (getenv ("OPENVPN_DEBUG"))
+		debug = TRUE;
+
+	if (debug)
+		g_message ("nm-openvpn-service (version " DIST_VERSION ") starting...");
+
 	if (system ("/sbin/modprobe tun") == -1)
 		exit (EXIT_FAILURE);
 
@@ -1190,15 +1389,15 @@ main (int argc, char *argv[])
 	if (!plugin)
 		exit (EXIT_FAILURE);
 
-	main_loop = g_main_loop_new (NULL, FALSE);
+	loop = g_main_loop_new (NULL, FALSE);
 
-	g_signal_connect (plugin, "quit",
-				   G_CALLBACK (quit_mainloop),
-				   main_loop);
+	if (!persist)
+		g_signal_connect (plugin, "quit", G_CALLBACK (quit_mainloop), loop);
 
-	g_main_loop_run (main_loop);
+	setup_signals ();
+	g_main_loop_run (loop);
 
-	g_main_loop_unref (main_loop);
+	g_main_loop_unref (loop);
 	g_object_unref (plugin);
 
 	exit (EXIT_SUCCESS);
